# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_facing_device_source_type import ClientFacingDeviceSourceType


class ClientFacingDevice(UniversalBaseModel):
    id: str
    user_id: str
    provider: str
    source_type: ClientFacingDeviceSourceType = pydantic.Field()
    """
    ℹ️ This enum is non-exhaustive.
    """

    app_id: typing.Optional[str] = None
    device_manufacturer: typing.Optional[str] = None
    device_model: typing.Optional[str] = None
    device_version: typing.Optional[str] = None
    device_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class LoincMatch(UniversalBaseModel):
    loinc_code: str
    loinc_name: str
    display_name: typing.Optional[str] = None
    aliases: typing.Optional[typing.List[str]] = None
    confidence_score: float

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .interpretation import Interpretation
from .lab_report_result_type import LabReportResultType
from .loinc_match import LoincMatch


class LabReportResult(UniversalBaseModel):
    test_name: str
    value: str
    type: typing.Optional[LabReportResultType] = None
    units: typing.Optional[str] = None
    max_reference_range: typing.Optional[float] = None
    min_reference_range: typing.Optional[float] = None
    source_panel_name: typing.Optional[str] = None
    loinc_matches: typing.Optional[typing.List[LoincMatch]] = None
    interpretation: typing.Optional[Interpretation] = None
    is_above_max_range: typing.Optional[bool] = None
    is_below_min_range: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    BulkExportLinkRequestTeamId,
    BulkImportLinkRequestTeamId,
    BulkPauseLinkRequestTeamId,
    BulkTriggerHistoricalPullLinkRequestTeamId,
    ListBulkOpsLinkRequestTeamId,
)

__all__ = [
    "BulkExportLinkRequestTeamId",
    "BulkImportLinkRequestTeamId",
    "BulkPauseLinkRequestTeamId",
    "BulkTriggerHistoricalPullLinkRequestTeamId",
    "ListBulkOpsLinkRequestTeamId",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.labs import Labs
from ...types.providers import Providers

GetUserHistoricalPullsIntrospectRequestProvider = typing.Union[Providers, Labs]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ClientFacingFallSample(UniversalBaseModel):
    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Deprecated
    """

    timezone_offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    Time zone UTC offset in seconds. Positive offset indicates east of UTC; negative offset indicates west of UTC; and null indicates the time zone information is unavailable at source.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type is always null for this resource.
    """

    unit: typing.Literal["count"] = "count"
    timestamp: dt.datetime = pydantic.Field()
    """
    Depracated. The start time (inclusive) of the interval.
    """

    start: dt.datetime = pydantic.Field()
    """
    The start time (inclusive) of the interval.
    """

    end: dt.datetime = pydantic.Field()
    """
    The end time (exclusive) of the interval.
    """

    value: float = pydantic.Field()
    """
    The recorded value for the interval.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_facing_heart_rate_timeseries import ClientFacingHeartRateTimeseries
from .client_facing_hrv_timeseries import ClientFacingHrvTimeseries
from .client_facing_hypnogram_timeseries import ClientFacingHypnogramTimeseries
from .client_facing_respiratory_rate_timeseries import ClientFacingRespiratoryRateTimeseries


class ClientFacingSleepStream(UniversalBaseModel):
    hrv: typing.Optional[typing.List[ClientFacingHrvTimeseries]] = None
    heartrate: typing.Optional[typing.List[ClientFacingHeartRateTimeseries]] = None
    hypnogram: typing.Optional[typing.List[ClientFacingHypnogramTimeseries]] = None
    respiratory_rate: typing.Optional[typing.List[ClientFacingRespiratoryRateTimeseries]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connected_source_client_facing import ConnectedSourceClientFacing
from .fallback_birth_date import FallbackBirthDate
from .fallback_time_zone import FallbackTimeZone


class ClientFacingUser(UniversalBaseModel):
    user_id: str = pydantic.Field()
    """
    User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.
    """

    team_id: str = pydantic.Field()
    """
    Your team id.
    """

    client_user_id: str = pydantic.Field()
    """
    A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.
    """

    created_on: dt.datetime = pydantic.Field()
    """
    When your item is created
    """

    connected_sources: typing.List[ConnectedSourceClientFacing] = pydantic.Field()
    """
    A list of the users connected sources.
    """

    fallback_time_zone: typing.Optional[FallbackTimeZone] = pydantic.Field(default=None)
    """
    
        Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
        Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).
        
    """

    fallback_birth_date: typing.Optional[FallbackBirthDate] = pydantic.Field(default=None)
    """
    Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.
    """

    ingestion_start: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    Starting bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).
    """

    ingestion_end: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    Ending bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.aggregation_response import AggregationResponse
from ..types.aggregation_result import AggregationResult
from ..types.continuous_query_task_history_response import ContinuousQueryTaskHistoryResponse
from ..types.query import Query
from ..types.query_config import QueryConfig
from .raw_client import AsyncRawAggregateClient, RawAggregateClient
from .types.query_batch_timeframe import QueryBatchTimeframe

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AggregateClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAggregateClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAggregateClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAggregateClient
        """
        return self._raw_client

    def query_one(
        self,
        user_id: str,
        *,
        timeframe: QueryBatchTimeframe,
        queries: typing.Sequence[Query],
        config: typing.Optional[QueryConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AggregationResponse:
        """
        Parameters
        ----------
        user_id : str

        timeframe : QueryBatchTimeframe

        queries : typing.Sequence[Query]

        config : typing.Optional[QueryConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AggregationResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import RelativeTimeframe
        import datetime
        from vital import Period
        from vital import PeriodUnit
        from vital import Query
        from vital import AggregateExpr
        from vital import SleepColumnExpr
        from vital import SleepColumnExprSleep
        from vital import AggregateExprFunc
        client = Vital(api_key="YOUR_API_KEY", )
        client.aggregate.query_one(user_id='user_id', timeframe=RelativeTimeframe(anchor=datetime.date.fromisoformat("2023-01-15", ), past=Period(unit=PeriodUnit.MINUTE, ), ), queries=[Query(select=[AggregateExpr(arg=SleepColumnExpr(sleep=SleepColumnExprSleep.ID, ), func=AggregateExprFunc.MEAN, )], )], )
        """
        _response = self._raw_client.query_one(
            user_id, timeframe=timeframe, queries=queries, config=config, request_options=request_options
        )
        return _response.data

    def get_result_table_for_continuous_query(
        self, user_id: str, query_id_or_slug: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AggregationResult:
        """
        Parameters
        ----------
        user_id : str

        query_id_or_slug : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AggregationResult
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.aggregate.get_result_table_for_continuous_query(user_id='user_id', query_id_or_slug='query_id_or_slug', )
        """
        _response = self._raw_client.get_result_table_for_continuous_query(
            user_id, query_id_or_slug, request_options=request_options
        )
        return _response.data

    def get_task_history_for_continuous_query(
        self,
        user_id: str,
        query_id_or_slug: str,
        *,
        next_cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContinuousQueryTaskHistoryResponse:
        """
        Parameters
        ----------
        user_id : str

        query_id_or_slug : str

        next_cursor : typing.Optional[str]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContinuousQueryTaskHistoryResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.aggregate.get_task_history_for_continuous_query(user_id='user_id', query_id_or_slug='query_id_or_slug', )
        """
        _response = self._raw_client.get_task_history_for_continuous_query(
            user_id, query_id_or_slug, next_cursor=next_cursor, limit=limit, request_options=request_options
        )
        return _response.data


class AsyncAggregateClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAggregateClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAggregateClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAggregateClient
        """
        return self._raw_client

    async def query_one(
        self,
        user_id: str,
        *,
        timeframe: QueryBatchTimeframe,
        queries: typing.Sequence[Query],
        config: typing.Optional[QueryConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AggregationResponse:
        """
        Parameters
        ----------
        user_id : str

        timeframe : QueryBatchTimeframe

        queries : typing.Sequence[Query]

        config : typing.Optional[QueryConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AggregationResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import RelativeTimeframe
        import datetime
        from vital import Period
        from vital import PeriodUnit
        from vital import Query
        from vital import AggregateExpr
        from vital import SleepColumnExpr
        from vital import SleepColumnExprSleep
        from vital import AggregateExprFunc
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.aggregate.query_one(user_id='user_id', timeframe=RelativeTimeframe(anchor=datetime.date.fromisoformat("2023-01-15", ), past=Period(unit=PeriodUnit.MINUTE, ), ), queries=[Query(select=[AggregateExpr(arg=SleepColumnExpr(sleep=SleepColumnExprSleep.ID, ), func=AggregateExprFunc.MEAN, )], )], )
        asyncio.run(main())
        """
        _response = await self._raw_client.query_one(
            user_id, timeframe=timeframe, queries=queries, config=config, request_options=request_options
        )
        return _response.data

    async def get_result_table_for_continuous_query(
        self, user_id: str, query_id_or_slug: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AggregationResult:
        """
        Parameters
        ----------
        user_id : str

        query_id_or_slug : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AggregationResult
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.aggregate.get_result_table_for_continuous_query(user_id='user_id', query_id_or_slug='query_id_or_slug', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_result_table_for_continuous_query(
            user_id, query_id_or_slug, request_options=request_options
        )
        return _response.data

    async def get_task_history_for_continuous_query(
        self,
        user_id: str,
        query_id_or_slug: str,
        *,
        next_cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContinuousQueryTaskHistoryResponse:
        """
        Parameters
        ----------
        user_id : str

        query_id_or_slug : str

        next_cursor : typing.Optional[str]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContinuousQueryTaskHistoryResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.aggregate.get_task_history_for_continuous_query(user_id='user_id', query_id_or_slug='query_id_or_slug', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_task_history_for_continuous_query(
            user_id, query_id_or_slug, next_cursor=next_cursor, limit=limit, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_destination_preferences import EventDestinationPreferences
from .libre_config import LibreConfig


class TeamConfig(UniversalBaseModel):
    libreview: LibreConfig
    texts_enabled: typing.Optional[bool] = None
    push_historical_data: typing.Optional[bool] = None
    provider_raw_data: typing.Optional[bool] = None
    reject_duplicate_connection: typing.Optional[bool] = None
    sdk_per_device_activity_timeseries: typing.Optional[bool] = None
    partial_webhooks_enabled: typing.Optional[bool] = None
    minor_ordering_enabled: typing.Optional[bool] = None
    eds_preferences: typing.Optional[EventDestinationPreferences] = None
    event_type_prefixes: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .parsed_lab_report_data import ParsedLabReportData
from .parsing_job_status import ParsingJobStatus


class ParsingJob(UniversalBaseModel):
    job_id: str
    status: ParsingJobStatus
    data: typing.Optional[ParsedLabReportData] = None
    needs_human_review: bool
    is_reviewed: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .get_lab_tests_request_order_direction import GetLabTestsRequestOrderDirection
from .get_lab_tests_request_order_key import GetLabTestsRequestOrderKey
from .get_orders_lab_tests_request_order_direction import GetOrdersLabTestsRequestOrderDirection
from .get_orders_lab_tests_request_order_key import GetOrdersLabTestsRequestOrderKey
from .get_paginated_lab_tests_request_order_direction import GetPaginatedLabTestsRequestOrderDirection
from .get_paginated_lab_tests_request_order_key import GetPaginatedLabTestsRequestOrderKey

__all__ = [
    "GetLabTestsRequestOrderDirection",
    "GetLabTestsRequestOrderKey",
    "GetOrdersLabTestsRequestOrderDirection",
    "GetOrdersLabTestsRequestOrderKey",
    "GetPaginatedLabTestsRequestOrderDirection",
    "GetPaginatedLabTestsRequestOrderKey",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ConnectionRecipe(UniversalBaseModel):
    user_id: str = pydantic.Field()
    """
    Vital User ID. The user must be created ahead of the bulk import operation.
    """

    access_token: str = pydantic.Field()
    """
    * OAuth 2.0 providers (Fitbit, etc): The latest Access Token.
    * OAuth 1.0 providers (Garmin): The Access Token.
    """

    refresh_token: str = pydantic.Field()
    """
    * OAuth 2.0 providers (Fitbit, etc): The latest Refresh Token.
    * OAuth 1.0 providers (Garmin): The Token Secret.
    """

    provider_id: str = pydantic.Field()
    """
    User ID of the data provider.
    
    * Fitbit: 6-character Fitbit User ID 
    * Garmin: 36-character Garmin User ID
    """

    expires_at: int = pydantic.Field()
    """
    Access token expiry date, in terms of UNIX epoch seconds.
    
    * OAuth 2.0 providers (Fitbit, etc): The latest expiry date on your record.
    * OAuth 1.0 providers (Garmin): Use the constant value `2147483647`.
    """

    oauth_scopes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    OAuth scopes of the data provider. Specify `null` if you do not
    have any scopes on record.
    
    * Fitbit: Has scopes
    * Garmin: No scope
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AppointmentStatus(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    CONFIRMED = "confirmed"
    PENDING = "pending"
    RESERVED = "reserved"
    IN_PROGRESS = "in_progress"
    COMPLETED = "completed"
    CANCELLED = "cancelled"
    _UNKNOWN = "__APPOINTMENTSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AppointmentStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        confirmed: typing.Callable[[], T_Result],
        pending: typing.Callable[[], T_Result],
        reserved: typing.Callable[[], T_Result],
        in_progress: typing.Callable[[], T_Result],
        completed: typing.Callable[[], T_Result],
        cancelled: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AppointmentStatus.CONFIRMED:
            return confirmed()
        if self is AppointmentStatus.PENDING:
            return pending()
        if self is AppointmentStatus.RESERVED:
            return reserved()
        if self is AppointmentStatus.IN_PROGRESS:
            return in_progress()
        if self is AppointmentStatus.COMPLETED:
            return completed()
        if self is AppointmentStatus.CANCELLED:
            return cancelled()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ParsingJobStatus(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    UPLOAD_PENDING = "upload_pending"
    STARTED = "started"
    COMPLETED = "completed"
    FAILED = "failed"
    _UNKNOWN = "__PARSINGJOBSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ParsingJobStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        upload_pending: typing.Callable[[], T_Result],
        started: typing.Callable[[], T_Result],
        completed: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ParsingJobStatus.UPLOAD_PENDING:
            return upload_pending()
        if self is ParsingJobStatus.STARTED:
            return started()
        if self is ParsingJobStatus.COMPLETED:
            return completed()
        if self is ParsingJobStatus.FAILED:
            return failed()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .bulk_export_link_request_team_id import BulkExportLinkRequestTeamId
from .bulk_import_link_request_team_id import BulkImportLinkRequestTeamId
from .bulk_pause_link_request_team_id import BulkPauseLinkRequestTeamId
from .bulk_trigger_historical_pull_link_request_team_id import BulkTriggerHistoricalPullLinkRequestTeamId
from .list_bulk_ops_link_request_team_id import ListBulkOpsLinkRequestTeamId

__all__ = [
    "BulkExportLinkRequestTeamId",
    "BulkImportLinkRequestTeamId",
    "BulkPauseLinkRequestTeamId",
    "BulkTriggerHistoricalPullLinkRequestTeamId",
    "ListBulkOpsLinkRequestTeamId",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    GetLabTestsRequestOrderDirection,
    GetLabTestsRequestOrderKey,
    GetOrdersLabTestsRequestOrderDirection,
    GetOrdersLabTestsRequestOrderKey,
    GetPaginatedLabTestsRequestOrderDirection,
    GetPaginatedLabTestsRequestOrderKey,
)

__all__ = [
    "GetLabTestsRequestOrderDirection",
    "GetLabTestsRequestOrderKey",
    "GetOrdersLabTestsRequestOrderDirection",
    "GetOrdersLabTestsRequestOrderKey",
    "GetPaginatedLabTestsRequestOrderDirection",
    "GetPaginatedLabTestsRequestOrderKey",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .lab_results_metadata import LabResultsMetadata
from .lab_results_raw_results import LabResultsRawResults
from .missing_biomarker_result import MissingBiomarkerResult
from .sample_data import SampleData


class LabResultsRaw(UniversalBaseModel):
    metadata: LabResultsMetadata
    results: LabResultsRawResults
    missing_results: typing.Optional[typing.List[MissingBiomarkerResult]] = None
    sample_information: typing.Optional[typing.Dict[str, typing.Optional[SampleData]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AllowedRadius(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    TEN = "10"
    TWENTY = "20"
    TWENTY_FIVE = "25"
    FIFTY = "50"
    ONE_HUNDRED = "100"
    _UNKNOWN = "__ALLOWEDRADIUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AllowedRadius":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        ten: typing.Callable[[], T_Result],
        twenty: typing.Callable[[], T_Result],
        twenty_five: typing.Callable[[], T_Result],
        fifty: typing.Callable[[], T_Result],
        one_hundred: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AllowedRadius.TEN:
            return ten()
        if self is AllowedRadius.TWENTY:
            return twenty()
        if self is AllowedRadius.TWENTY_FIVE:
            return twenty_five()
        if self is AllowedRadius.FIFTY:
            return fifty()
        if self is AllowedRadius.ONE_HUNDRED:
            return one_hundred()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_facing_lab import ClientFacingLab
from .client_facing_marker import ClientFacingMarker
from .lab_test_collection_method import LabTestCollectionMethod
from .lab_test_sample_type import LabTestSampleType
from .lab_test_status import LabTestStatus


class ClientFacingLabTest(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    
    """

    slug: str = pydantic.Field()
    """
    
    """

    name: str = pydantic.Field()
    """
    
    """

    sample_type: LabTestSampleType = pydantic.Field()
    """
    
    """

    method: LabTestCollectionMethod = pydantic.Field()
    """
    
    """

    price: float = pydantic.Field()
    """
    
    """

    is_active: bool = pydantic.Field()
    """
    Deprecated. Use status instead.
    """

    status: LabTestStatus = pydantic.Field()
    """
    
    """

    fasting: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Defines whether a lab test requires fasting.
    """

    lab: typing.Optional[ClientFacingLab] = pydantic.Field(default=None)
    """
    
    """

    markers: typing.Optional[typing.List[ClientFacingMarker]] = pydantic.Field(default=None)
    """
    
    """

    is_delegated: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Deprecated and always false. Delegation is now at the lab account level. Used to denote whether a lab test requires using non-Vital physician networks.
    """

    auto_generated: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the lab test was auto-generated by Vital
    """

    has_collection_instructions: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether or not the lab test has collection instructions.
    """

    common_tat_days: typing.Optional[int] = pydantic.Field(default=None)
    """
    The common turnaround time in days for the lab test. This is the expected time for the lab to process the test and return results.
    """

    worst_case_tat_days: typing.Optional[int] = pydantic.Field(default=None)
    """
    The worst-case turnaround time in days for the lab test. This is the maximum time the lab may take to process the test and return results.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .query_group_by_item import QueryGroupByItem
from .query_select_item import QuerySelectItem


class Query(UniversalBaseModel):
    select: typing.List[QuerySelectItem]
    group_by: typing.Optional[typing.List[QueryGroupByItem]] = None
    where: typing.Optional[str] = pydantic.Field(default=None)
    """
    A WHERE clause filtering the input data. If a GROUP BY clause is present, filtering happens prior to GROUP BY evaluation.
    
    WHERE clause uses SQL Expression syntax to describe the filtering criteria:
    * Available operators: `>`, `>=`, `<`, `<=`, `=`, `!=`, `NOT`, `AND` and `OR`.
    * Parentheses is supported.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TraceElements(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    CHROMIUM = "chromium"
    COPPER = "copper"
    IODINE = "iodine"
    MANGANESE = "manganese"
    MOLYBDENUM = "molybdenum"
    SELENIUM = "selenium"
    _UNKNOWN = "__TRACEELEMENTS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "TraceElements":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        chromium: typing.Callable[[], T_Result],
        copper: typing.Callable[[], T_Result],
        iodine: typing.Callable[[], T_Result],
        manganese: typing.Callable[[], T_Result],
        molybdenum: typing.Callable[[], T_Result],
        selenium: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is TraceElements.CHROMIUM:
            return chromium()
        if self is TraceElements.COPPER:
            return copper()
        if self is TraceElements.IODINE:
            return iodine()
        if self is TraceElements.MANGANESE:
            return manganese()
        if self is TraceElements.MOLYBDENUM:
            return molybdenum()
        if self is TraceElements.SELENIUM:
            return selenium()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Fats(UniversalBaseModel):
    saturated: typing.Optional[float] = pydantic.Field(default=None)
    """
    Amount of saturated fats in grams (g)
    """

    monounsaturated: typing.Optional[float] = pydantic.Field(default=None)
    """
    Amount of monounsaturated fats in grams (g)
    """

    polyunsaturated: typing.Optional[float] = pydantic.Field(default=None)
    """
    Amount of polyunsaturated fats in grams (g)
    """

    omega_3: typing.Optional[float] = pydantic.Field(alias="omega3", default=None)
    """
    Amount of Omega-3 fatty acids in grams (g)
    """

    omega_6: typing.Optional[float] = pydantic.Field(alias="omega6", default=None)
    """
    Amount of Omega-6 fatty acids in grams (g)
    """

    total: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total amount of fats in grams (g)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.parsing_job import ParsingJob
from .raw_client import AsyncRawLabReportClient, RawLabReportClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LabReportClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLabReportClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLabReportClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLabReportClient
        """
        return self._raw_client

    def parser_create_job(
        self,
        *,
        file: core.File,
        needs_human_review: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ParsingJob:
        """
        Creates a parse job, uploads the file to provider, persists the job row,
        and starts the ParseLabReport. Returns a generated job_id.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        needs_human_review : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ParsingJob
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.lab_report.parser_create_job()
        """
        _response = self._raw_client.parser_create_job(
            file=file, needs_human_review=needs_human_review, request_options=request_options
        )
        return _response.data

    def parser_get_job(self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ParsingJob:
        """
        Retrieves the parse job status and stored result if completed.

        Returns:
            ParseLabResultJobResponse with job status and parsed data (if complete)

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ParsingJob
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.lab_report.parser_get_job(job_id='job_id', )
        """
        _response = self._raw_client.parser_get_job(job_id, request_options=request_options)
        return _response.data


class AsyncLabReportClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLabReportClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLabReportClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLabReportClient
        """
        return self._raw_client

    async def parser_create_job(
        self,
        *,
        file: core.File,
        needs_human_review: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ParsingJob:
        """
        Creates a parse job, uploads the file to provider, persists the job row,
        and starts the ParseLabReport. Returns a generated job_id.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        needs_human_review : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ParsingJob
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.lab_report.parser_create_job()
        asyncio.run(main())
        """
        _response = await self._raw_client.parser_create_job(
            file=file, needs_human_review=needs_human_review, request_options=request_options
        )
        return _response.data

    async def parser_get_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ParsingJob:
        """
        Retrieves the parse job status and stored result if completed.

        Returns:
            ParseLabResultJobResponse with job status and parsed data (if complete)

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ParsingJob
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.lab_report.parser_get_job(job_id='job_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.parser_get_job(job_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.client_facing_physician import ClientFacingPhysician
from ..types.client_facing_team import ClientFacingTeam
from ..types.client_facing_user import ClientFacingUser
from ..types.http_validation_error import HttpValidationError
from ..types.priority_resource import PriorityResource


class RawTeamClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_link_config(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]:
        """
        Post teams.

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/team/link/config",
            method="GET",
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, team_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ClientFacingTeam]:
        """
        Get team.

        Parameters
        ----------
        team_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ClientFacingTeam]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/team/{jsonable_encoder(team_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ClientFacingTeam,
                    parse_obj_as(
                        type_=ClientFacingTeam,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_user_by_id(
        self, *, query_id: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[ClientFacingUser]]:
        """
        Search team users by user_id

        Parameters
        ----------
        query_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[ClientFacingUser]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/team/users/search",
            method="GET",
            params={
                "query_id": query_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[ClientFacingUser],
                    parse_obj_as(
                        type_=typing.List[ClientFacingUser],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_svix_url(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/team/svix/url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_source_priorities(
        self,
        *,
        data_type: typing.Optional[PriorityResource] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]:
        """
        GET source priorities.

        Parameters
        ----------
        data_type : typing.Optional[PriorityResource]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/team/source/priorities",
            method="GET",
            params={
                "data_type": data_type,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[typing.Dict[str, typing.Optional[typing.Any]]],
                    parse_obj_as(
                        type_=typing.List[typing.Dict[str, typing.Optional[typing.Any]]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_source_priorities(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]:
        """
        Patch source priorities.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/team/source/priorities",
            method="PATCH",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[typing.Dict[str, typing.Optional[typing.Any]]],
                    parse_obj_as(
                        type_=typing.List[typing.Dict[str, typing.Optional[typing.Any]]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_physicians(
        self, team_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[ClientFacingPhysician]]:
        """
        Parameters
        ----------
        team_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[ClientFacingPhysician]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/team/{jsonable_encoder(team_id)}/physicians",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[ClientFacingPhysician],
                    parse_obj_as(
                        type_=typing.List[ClientFacingPhysician],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTeamClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_link_config(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]:
        """
        Post teams.

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/team/link/config",
            method="GET",
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, team_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ClientFacingTeam]:
        """
        Get team.

        Parameters
        ----------
        team_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ClientFacingTeam]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/team/{jsonable_encoder(team_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ClientFacingTeam,
                    parse_obj_as(
                        type_=ClientFacingTeam,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_user_by_id(
        self, *, query_id: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[ClientFacingUser]]:
        """
        Search team users by user_id

        Parameters
        ----------
        query_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[ClientFacingUser]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/team/users/search",
            method="GET",
            params={
                "query_id": query_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[ClientFacingUser],
                    parse_obj_as(
                        type_=typing.List[ClientFacingUser],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_svix_url(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/team/svix/url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_source_priorities(
        self,
        *,
        data_type: typing.Optional[PriorityResource] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]:
        """
        GET source priorities.

        Parameters
        ----------
        data_type : typing.Optional[PriorityResource]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/team/source/priorities",
            method="GET",
            params={
                "data_type": data_type,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[typing.Dict[str, typing.Optional[typing.Any]]],
                    parse_obj_as(
                        type_=typing.List[typing.Dict[str, typing.Optional[typing.Any]]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_source_priorities(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]:
        """
        Patch source priorities.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/team/source/priorities",
            method="PATCH",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[typing.Dict[str, typing.Optional[typing.Any]]],
                    parse_obj_as(
                        type_=typing.List[typing.Dict[str, typing.Optional[typing.Any]]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_physicians(
        self, team_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[ClientFacingPhysician]]:
        """
        Parameters
        ----------
        team_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[ClientFacingPhysician]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/team/{jsonable_encoder(team_id)}/physicians",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[ClientFacingPhysician],
                    parse_obj_as(
                        type_=typing.List[ClientFacingPhysician],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .. import core
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.parsing_job import ParsingJob

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLabReportClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def parser_create_job(
        self,
        *,
        file: core.File,
        needs_human_review: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ParsingJob]:
        """
        Creates a parse job, uploads the file to provider, persists the job row,
        and starts the ParseLabReport. Returns a generated job_id.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        needs_human_review : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ParsingJob]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "lab_report/v1/parser/job",
            method="POST",
            data={
                "needs_human_review": needs_human_review,
            },
            files={
                "file": file,
            },
            headers={
                "content-type": "multipart/form-data",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ParsingJob,
                    parse_obj_as(
                        type_=ParsingJob,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def parser_get_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ParsingJob]:
        """
        Retrieves the parse job status and stored result if completed.

        Returns:
            ParseLabResultJobResponse with job status and parsed data (if complete)

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ParsingJob]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"lab_report/v1/parser/job/{jsonable_encoder(job_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ParsingJob,
                    parse_obj_as(
                        type_=ParsingJob,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLabReportClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def parser_create_job(
        self,
        *,
        file: core.File,
        needs_human_review: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ParsingJob]:
        """
        Creates a parse job, uploads the file to provider, persists the job row,
        and starts the ParseLabReport. Returns a generated job_id.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        needs_human_review : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ParsingJob]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "lab_report/v1/parser/job",
            method="POST",
            data={
                "needs_human_review": needs_human_review,
            },
            files={
                "file": file,
            },
            headers={
                "content-type": "multipart/form-data",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ParsingJob,
                    parse_obj_as(
                        type_=ParsingJob,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def parser_get_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ParsingJob]:
        """
        Retrieves the parse job status and stored result if completed.

        Returns:
            ParseLabResultJobResponse with job status and parsed data (if complete)

        Parameters
        ----------
        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ParsingJob]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"lab_report/v1/parser/job/{jsonable_encoder(job_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ParsingJob,
                    parse_obj_as(
                        type_=ParsingJob,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.auth_type import AuthType
from ..types.bulk_export_connections_response import BulkExportConnectionsResponse
from ..types.bulk_import_connections_response import BulkImportConnectionsResponse
from ..types.bulk_ops_response import BulkOpsResponse
from ..types.connection_recipe import ConnectionRecipe
from ..types.demo_connection_status import DemoConnectionStatus
from ..types.demo_providers import DemoProviders
from ..types.link_token_exchange_response import LinkTokenExchangeResponse
from ..types.manual_providers import ManualProviders
from ..types.o_auth_providers import OAuthProviders
from ..types.password_providers import PasswordProviders
from ..types.provider_link_response import ProviderLinkResponse
from ..types.providers import Providers
from ..types.region import Region
from ..types.source import Source
from ..types.source_link import SourceLink
from ..types.vital_token_created_response import VitalTokenCreatedResponse
from .raw_client import AsyncRawLinkClient, RawLinkClient
from .types.bulk_export_link_request_team_id import BulkExportLinkRequestTeamId
from .types.bulk_import_link_request_team_id import BulkImportLinkRequestTeamId
from .types.bulk_pause_link_request_team_id import BulkPauseLinkRequestTeamId
from .types.bulk_trigger_historical_pull_link_request_team_id import BulkTriggerHistoricalPullLinkRequestTeamId
from .types.list_bulk_ops_link_request_team_id import ListBulkOpsLinkRequestTeamId

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LinkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLinkClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLinkClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLinkClient
        """
        return self._raw_client

    def list_bulk_ops(
        self,
        *,
        next_cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        team_id: typing.Optional[ListBulkOpsLinkRequestTeamId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkOpsResponse:
        """
        Parameters
        ----------
        next_cursor : typing.Optional[str]

        page_size : typing.Optional[int]

        team_id : typing.Optional[ListBulkOpsLinkRequestTeamId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkOpsResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.list_bulk_ops()
        """
        _response = self._raw_client.list_bulk_ops(
            next_cursor=next_cursor, page_size=page_size, team_id=team_id, request_options=request_options
        )
        return _response.data

    def bulk_import(
        self,
        *,
        provider: OAuthProviders,
        connections: typing.Sequence[ConnectionRecipe],
        team_id: typing.Optional[BulkImportLinkRequestTeamId] = None,
        wait_for_completion: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkImportConnectionsResponse:
        """
        Parameters
        ----------
        provider : OAuthProviders

        connections : typing.Sequence[ConnectionRecipe]

        team_id : typing.Optional[BulkImportLinkRequestTeamId]

        wait_for_completion : typing.Optional[bool]

            Whether or not the endpoint should wait for the Bulk Op to complete before responding.

            When `wait_for_completion` is enabled, the endpoint may respond 200 OK if the Bulk Op takes less than 20 seconds to complete.

            Otherwise, the endpoint always responds with 202 Created once the submitted data have been enqueued successfully. You can use
            the [List Bulk Ops](https://docs.tryvital.io/api-reference/link/list-bulk-ops) endpoint to inspect the progress of the Bulk Op.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkImportConnectionsResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import OAuthProviders
        from vital import ConnectionRecipe
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.bulk_import(provider=OAuthProviders.OURA, connections=[ConnectionRecipe(user_id='user_id', access_token='access_token', refresh_token='refresh_token', provider_id='provider_id', expires_at=1, )], )
        """
        _response = self._raw_client.bulk_import(
            provider=provider,
            connections=connections,
            team_id=team_id,
            wait_for_completion=wait_for_completion,
            request_options=request_options,
        )
        return _response.data

    def bulk_trigger_historical_pull(
        self,
        *,
        user_ids: typing.Sequence[str],
        provider: OAuthProviders,
        team_id: typing.Optional[BulkTriggerHistoricalPullLinkRequestTeamId] = None,
        wait_for_completion: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        user_ids : typing.Sequence[str]

        provider : OAuthProviders

        team_id : typing.Optional[BulkTriggerHistoricalPullLinkRequestTeamId]

        wait_for_completion : typing.Optional[bool]

            Whether or not the endpoint should wait for the Bulk Op to complete before responding.

            When `wait_for_completion` is enabled, the endpoint may respond 200 OK if the Bulk Op takes less than 20 seconds to complete.

            Otherwise, the endpoint always responds with 202 Created once the submitted data have been enqueued successfully. You can use
            the [List Bulk Ops](https://docs.tryvital.io/api-reference/link/list-bulk-ops) endpoint to inspect the progress of the Bulk Op.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import OAuthProviders
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.bulk_trigger_historical_pull(user_ids=['user_ids'], provider=OAuthProviders.OURA, )
        """
        _response = self._raw_client.bulk_trigger_historical_pull(
            user_ids=user_ids,
            provider=provider,
            team_id=team_id,
            wait_for_completion=wait_for_completion,
            request_options=request_options,
        )
        return _response.data

    def bulk_export(
        self,
        *,
        provider: OAuthProviders,
        team_id: typing.Optional[BulkExportLinkRequestTeamId] = None,
        user_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        next_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkExportConnectionsResponse:
        """
        Parameters
        ----------
        provider : OAuthProviders

        team_id : typing.Optional[BulkExportLinkRequestTeamId]

        user_ids : typing.Optional[typing.Sequence[str]]

        next_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkExportConnectionsResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import OAuthProviders
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.bulk_export(provider=OAuthProviders.OURA, )
        """
        _response = self._raw_client.bulk_export(
            provider=provider,
            team_id=team_id,
            user_ids=user_ids,
            next_token=next_token,
            request_options=request_options,
        )
        return _response.data

    def bulk_pause(
        self,
        *,
        user_ids: typing.Sequence[str],
        provider: OAuthProviders,
        team_id: typing.Optional[BulkPauseLinkRequestTeamId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        user_ids : typing.Sequence[str]

        provider : OAuthProviders

        team_id : typing.Optional[BulkPauseLinkRequestTeamId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import OAuthProviders
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.bulk_pause(user_ids=['user_ids'], provider=OAuthProviders.OURA, )
        """
        _response = self._raw_client.bulk_pause(
            user_ids=user_ids, provider=provider, team_id=team_id, request_options=request_options
        )
        return _response.data

    def token(
        self,
        *,
        user_id: str,
        provider: typing.Optional[Providers] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        filter_on_providers: typing.Optional[typing.Sequence[Providers]] = OMIT,
        on_error: typing.Optional[typing.Literal["redirect"]] = OMIT,
        on_close: typing.Optional[typing.Literal["redirect"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LinkTokenExchangeResponse:
        """
        Endpoint to generate a user link token, to be used throughout the vital
        link process. The vital link token is a one time use token, that
        expires after 10 minutes. If you would like vital-link widget to launch
        with a specific provider, pass in a provider in the body. If you would
        like to redirect to a custom url after successful or error connection,
        pass in your own custom redirect_url parameter.

        Parameters
        ----------
        user_id : str
            User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.

        provider : typing.Optional[Providers]

        redirect_url : typing.Optional[str]

        filter_on_providers : typing.Optional[typing.Sequence[Providers]]
            An allowlist of providers dictating what Vital Link Widget should show to the end user.
            If unspecified, all linkable providers are shown.

            This has no effect on programmatic Vital Link API usage.

        on_error : typing.Optional[typing.Literal["redirect"]]
            By default, Vital Link Widget input forms for password and email providers have in-built error handling.

            Specifying `on_error=redirect` disables this Vital Link Widget UI behaviour — it would
            instead redirect to your `redirect_url`, with Link Error parameters injected.

            This has no effect on OAuth providers — they always redirect to your `redirect_url`. This also has
            no effect on programmatic Vital Link API usage.

        on_close : typing.Optional[typing.Literal["redirect"]]
            By default, Vital Link Widget closes the window or tab when the user taps the Close button.

            Specifying `on_close=redirect` would change the Close button behaviour to redirect to your `redirect_url`
            with the `user_cancelled` error specified.

            This has no effect on programmatic Vital Link API usage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LinkTokenExchangeResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.token(user_id='user_id', )
        """
        _response = self._raw_client.token(
            user_id=user_id,
            provider=provider,
            redirect_url=redirect_url,
            filter_on_providers=filter_on_providers,
            on_error=on_error,
            on_close=on_close,
            request_options=request_options,
        )
        return _response.data

    def is_token_valid(
        self, *, token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        token : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.is_token_valid(token='token', )
        """
        _response = self._raw_client.is_token_valid(token=token, request_options=request_options)
        return _response.data

    def code_create(
        self,
        *,
        user_id: str,
        expires_at: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VitalTokenCreatedResponse:
        """
        Generate a token to invite a user of Vital mobile app to your team

        Parameters
        ----------
        user_id : str

        expires_at : typing.Optional[dt.datetime]
            When the link code should expire. Defaults to server time plus 1 hour.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VitalTokenCreatedResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.code_create(user_id='user_id', )
        """
        _response = self._raw_client.code_create(
            user_id=user_id, expires_at=expires_at, request_options=request_options
        )
        return _response.data

    def start_connect(
        self, *, link_token: str, provider: Providers, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        REQUEST_SOURCE: VITAL-LINK
        Start link token process

        Parameters
        ----------
        link_token : str

        provider : Providers

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import Providers
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.start_connect(link_token='link_token', provider=Providers.OURA, )
        """
        _response = self._raw_client.start_connect(
            link_token=link_token, provider=provider, request_options=request_options
        )
        return _response.data

    def token_state(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        REQUEST_SOURCE: VITAL-LINK
        Check link token state - can be hit continuously used as heartbeat

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.token_state()
        """
        _response = self._raw_client.token_state(vital_link_token=vital_link_token, request_options=request_options)
        return _response.data

    def email_auth(
        self,
        *,
        email: str,
        provider: Providers,
        auth_type: AuthType,
        vital_link_token: typing.Optional[str] = None,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Deprecated. Use `POST /v2/link/provider/email/{provider}` instead.

        Parameters
        ----------
        email : str

        provider : Providers

        auth_type : AuthType

        vital_link_token : typing.Optional[str]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import Providers
        from vital import AuthType
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.email_auth(email='email', provider=Providers.OURA, auth_type=AuthType.PASSWORD, )
        """
        _response = self._raw_client.email_auth(
            email=email,
            provider=provider,
            auth_type=auth_type,
            vital_link_token=vital_link_token,
            region=region,
            request_options=request_options,
        )
        return _response.data

    def password_auth(
        self,
        *,
        username: str,
        password: str,
        provider: Providers,
        auth_type: AuthType,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Deprecated. Use `POST /v2/link/provider/password/{provider}` instead.

        Parameters
        ----------
        username : str

        password : str

        provider : Providers

        auth_type : AuthType

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import Providers
        from vital import AuthType
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.password_auth(username='username', password='password', provider=Providers.OURA, auth_type=AuthType.PASSWORD, )
        """
        _response = self._raw_client.password_auth(
            username=username,
            password=password,
            provider=provider,
            auth_type=auth_type,
            vital_link_token=vital_link_token,
            request_options=request_options,
        )
        return _response.data

    def generate_oauth_link(
        self,
        oauth_provider: OAuthProviders,
        *,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Source:
        """
        This endpoint generates an OAuth link for oauth provider

        Parameters
        ----------
        oauth_provider : OAuthProviders

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Source
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import OAuthProviders
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.generate_oauth_link(oauth_provider=OAuthProviders.OURA, )
        """
        _response = self._raw_client.generate_oauth_link(
            oauth_provider, vital_link_token=vital_link_token, request_options=request_options
        )
        return _response.data

    def connect_password_provider(
        self,
        provider: PasswordProviders,
        *,
        username: str,
        password: str,
        vital_link_token: typing.Optional[str] = None,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProviderLinkResponse:
        """
        This connects auth providers that are password based.

        Parameters
        ----------
        provider : PasswordProviders

        username : str
            Username for provider

        password : str
            Password for provider

        vital_link_token : typing.Optional[str]

        region : typing.Optional[Region]
            Provider region to authenticate against. Only applicable to specific providers.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProviderLinkResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import PasswordProviders
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.connect_password_provider(provider=PasswordProviders.WHOOP, username='username', password='password', )
        """
        _response = self._raw_client.connect_password_provider(
            provider,
            username=username,
            password=password,
            vital_link_token=vital_link_token,
            region=region,
            request_options=request_options,
        )
        return _response.data

    def complete_password_provider_mfa(
        self,
        provider: PasswordProviders,
        *,
        mfa_code: str,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProviderLinkResponse:
        """
        This connects auth providers that are password based.

        Parameters
        ----------
        provider : PasswordProviders

        mfa_code : str

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProviderLinkResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import PasswordProviders
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.complete_password_provider_mfa(provider=PasswordProviders.WHOOP, mfa_code='mfa_code', )
        """
        _response = self._raw_client.complete_password_provider_mfa(
            provider, mfa_code=mfa_code, vital_link_token=vital_link_token, request_options=request_options
        )
        return _response.data

    def connect_email_auth_provider(
        self,
        *,
        email: str,
        vital_link_token: typing.Optional[str] = None,
        email_provider_auth_link_provider: typing.Optional[Providers] = OMIT,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        This connects auth providers that are email based.

        Parameters
        ----------
        email : str

        vital_link_token : typing.Optional[str]

        email_provider_auth_link_provider : typing.Optional[Providers]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.connect_email_auth_provider(email='email', )
        """
        _response = self._raw_client.connect_email_auth_provider(
            email=email,
            vital_link_token=vital_link_token,
            email_provider_auth_link_provider=email_provider_auth_link_provider,
            region=region,
            request_options=request_options,
        )
        return _response.data

    def get_all_providers(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[SourceLink]:
        """
        GET List of all available providers given the generated link token.

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceLink]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.get_all_providers()
        """
        _response = self._raw_client.get_all_providers(
            vital_link_token=vital_link_token, request_options=request_options
        )
        return _response.data

    def connect_manual_provider(
        self,
        provider: ManualProviders,
        *,
        user_id: str,
        provider_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, bool]:
        """
        Parameters
        ----------
        provider : ManualProviders

        user_id : str

        provider_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, bool]
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import ManualProviders
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.connect_manual_provider(provider=ManualProviders.BEURER_BLE, user_id='user_id', )
        """
        _response = self._raw_client.connect_manual_provider(
            provider, user_id=user_id, provider_id=provider_id, request_options=request_options
        )
        return _response.data

    def connect_demo_provider(
        self, *, user_id: str, provider: DemoProviders, request_options: typing.Optional[RequestOptions] = None
    ) -> DemoConnectionStatus:
        """
        POST Connect the given Vital user to a demo provider.

        Parameters
        ----------
        user_id : str
            Vital user ID

        provider : DemoProviders
            Demo provider. For more information, please check out our docs (https://docs.tryvital.io/wearables/providers/test_data)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DemoConnectionStatus
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import DemoProviders
        client = Vital(api_key="YOUR_API_KEY", )
        client.link.connect_demo_provider(user_id='user_id', provider=DemoProviders.APPLE_HEALTH_KIT, )
        """
        _response = self._raw_client.connect_demo_provider(
            user_id=user_id, provider=provider, request_options=request_options
        )
        return _response.data


class AsyncLinkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLinkClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLinkClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLinkClient
        """
        return self._raw_client

    async def list_bulk_ops(
        self,
        *,
        next_cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        team_id: typing.Optional[ListBulkOpsLinkRequestTeamId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkOpsResponse:
        """
        Parameters
        ----------
        next_cursor : typing.Optional[str]

        page_size : typing.Optional[int]

        team_id : typing.Optional[ListBulkOpsLinkRequestTeamId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkOpsResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.list_bulk_ops()
        asyncio.run(main())
        """
        _response = await self._raw_client.list_bulk_ops(
            next_cursor=next_cursor, page_size=page_size, team_id=team_id, request_options=request_options
        )
        return _response.data

    async def bulk_import(
        self,
        *,
        provider: OAuthProviders,
        connections: typing.Sequence[ConnectionRecipe],
        team_id: typing.Optional[BulkImportLinkRequestTeamId] = None,
        wait_for_completion: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkImportConnectionsResponse:
        """
        Parameters
        ----------
        provider : OAuthProviders

        connections : typing.Sequence[ConnectionRecipe]

        team_id : typing.Optional[BulkImportLinkRequestTeamId]

        wait_for_completion : typing.Optional[bool]

            Whether or not the endpoint should wait for the Bulk Op to complete before responding.

            When `wait_for_completion` is enabled, the endpoint may respond 200 OK if the Bulk Op takes less than 20 seconds to complete.

            Otherwise, the endpoint always responds with 202 Created once the submitted data have been enqueued successfully. You can use
            the [List Bulk Ops](https://docs.tryvital.io/api-reference/link/list-bulk-ops) endpoint to inspect the progress of the Bulk Op.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkImportConnectionsResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import OAuthProviders
        from vital import ConnectionRecipe
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.bulk_import(provider=OAuthProviders.OURA, connections=[ConnectionRecipe(user_id='user_id', access_token='access_token', refresh_token='refresh_token', provider_id='provider_id', expires_at=1, )], )
        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_import(
            provider=provider,
            connections=connections,
            team_id=team_id,
            wait_for_completion=wait_for_completion,
            request_options=request_options,
        )
        return _response.data

    async def bulk_trigger_historical_pull(
        self,
        *,
        user_ids: typing.Sequence[str],
        provider: OAuthProviders,
        team_id: typing.Optional[BulkTriggerHistoricalPullLinkRequestTeamId] = None,
        wait_for_completion: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        user_ids : typing.Sequence[str]

        provider : OAuthProviders

        team_id : typing.Optional[BulkTriggerHistoricalPullLinkRequestTeamId]

        wait_for_completion : typing.Optional[bool]

            Whether or not the endpoint should wait for the Bulk Op to complete before responding.

            When `wait_for_completion` is enabled, the endpoint may respond 200 OK if the Bulk Op takes less than 20 seconds to complete.

            Otherwise, the endpoint always responds with 202 Created once the submitted data have been enqueued successfully. You can use
            the [List Bulk Ops](https://docs.tryvital.io/api-reference/link/list-bulk-ops) endpoint to inspect the progress of the Bulk Op.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import OAuthProviders
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.bulk_trigger_historical_pull(user_ids=['user_ids'], provider=OAuthProviders.OURA, )
        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_trigger_historical_pull(
            user_ids=user_ids,
            provider=provider,
            team_id=team_id,
            wait_for_completion=wait_for_completion,
            request_options=request_options,
        )
        return _response.data

    async def bulk_export(
        self,
        *,
        provider: OAuthProviders,
        team_id: typing.Optional[BulkExportLinkRequestTeamId] = None,
        user_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        next_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkExportConnectionsResponse:
        """
        Parameters
        ----------
        provider : OAuthProviders

        team_id : typing.Optional[BulkExportLinkRequestTeamId]

        user_ids : typing.Optional[typing.Sequence[str]]

        next_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkExportConnectionsResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import OAuthProviders
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.bulk_export(provider=OAuthProviders.OURA, )
        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_export(
            provider=provider,
            team_id=team_id,
            user_ids=user_ids,
            next_token=next_token,
            request_options=request_options,
        )
        return _response.data

    async def bulk_pause(
        self,
        *,
        user_ids: typing.Sequence[str],
        provider: OAuthProviders,
        team_id: typing.Optional[BulkPauseLinkRequestTeamId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        user_ids : typing.Sequence[str]

        provider : OAuthProviders

        team_id : typing.Optional[BulkPauseLinkRequestTeamId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import OAuthProviders
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.bulk_pause(user_ids=['user_ids'], provider=OAuthProviders.OURA, )
        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_pause(
            user_ids=user_ids, provider=provider, team_id=team_id, request_options=request_options
        )
        return _response.data

    async def token(
        self,
        *,
        user_id: str,
        provider: typing.Optional[Providers] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        filter_on_providers: typing.Optional[typing.Sequence[Providers]] = OMIT,
        on_error: typing.Optional[typing.Literal["redirect"]] = OMIT,
        on_close: typing.Optional[typing.Literal["redirect"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LinkTokenExchangeResponse:
        """
        Endpoint to generate a user link token, to be used throughout the vital
        link process. The vital link token is a one time use token, that
        expires after 10 minutes. If you would like vital-link widget to launch
        with a specific provider, pass in a provider in the body. If you would
        like to redirect to a custom url after successful or error connection,
        pass in your own custom redirect_url parameter.

        Parameters
        ----------
        user_id : str
            User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.

        provider : typing.Optional[Providers]

        redirect_url : typing.Optional[str]

        filter_on_providers : typing.Optional[typing.Sequence[Providers]]
            An allowlist of providers dictating what Vital Link Widget should show to the end user.
            If unspecified, all linkable providers are shown.

            This has no effect on programmatic Vital Link API usage.

        on_error : typing.Optional[typing.Literal["redirect"]]
            By default, Vital Link Widget input forms for password and email providers have in-built error handling.

            Specifying `on_error=redirect` disables this Vital Link Widget UI behaviour — it would
            instead redirect to your `redirect_url`, with Link Error parameters injected.

            This has no effect on OAuth providers — they always redirect to your `redirect_url`. This also has
            no effect on programmatic Vital Link API usage.

        on_close : typing.Optional[typing.Literal["redirect"]]
            By default, Vital Link Widget closes the window or tab when the user taps the Close button.

            Specifying `on_close=redirect` would change the Close button behaviour to redirect to your `redirect_url`
            with the `user_cancelled` error specified.

            This has no effect on programmatic Vital Link API usage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LinkTokenExchangeResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.token(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.token(
            user_id=user_id,
            provider=provider,
            redirect_url=redirect_url,
            filter_on_providers=filter_on_providers,
            on_error=on_error,
            on_close=on_close,
            request_options=request_options,
        )
        return _response.data

    async def is_token_valid(
        self, *, token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        token : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.is_token_valid(token='token', )
        asyncio.run(main())
        """
        _response = await self._raw_client.is_token_valid(token=token, request_options=request_options)
        return _response.data

    async def code_create(
        self,
        *,
        user_id: str,
        expires_at: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VitalTokenCreatedResponse:
        """
        Generate a token to invite a user of Vital mobile app to your team

        Parameters
        ----------
        user_id : str

        expires_at : typing.Optional[dt.datetime]
            When the link code should expire. Defaults to server time plus 1 hour.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VitalTokenCreatedResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.code_create(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.code_create(
            user_id=user_id, expires_at=expires_at, request_options=request_options
        )
        return _response.data

    async def start_connect(
        self, *, link_token: str, provider: Providers, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        REQUEST_SOURCE: VITAL-LINK
        Start link token process

        Parameters
        ----------
        link_token : str

        provider : Providers

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import Providers
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.start_connect(link_token='link_token', provider=Providers.OURA, )
        asyncio.run(main())
        """
        _response = await self._raw_client.start_connect(
            link_token=link_token, provider=provider, request_options=request_options
        )
        return _response.data

    async def token_state(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        REQUEST_SOURCE: VITAL-LINK
        Check link token state - can be hit continuously used as heartbeat

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.token_state()
        asyncio.run(main())
        """
        _response = await self._raw_client.token_state(
            vital_link_token=vital_link_token, request_options=request_options
        )
        return _response.data

    async def email_auth(
        self,
        *,
        email: str,
        provider: Providers,
        auth_type: AuthType,
        vital_link_token: typing.Optional[str] = None,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Deprecated. Use `POST /v2/link/provider/email/{provider}` instead.

        Parameters
        ----------
        email : str

        provider : Providers

        auth_type : AuthType

        vital_link_token : typing.Optional[str]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import Providers
        from vital import AuthType
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.email_auth(email='email', provider=Providers.OURA, auth_type=AuthType.PASSWORD, )
        asyncio.run(main())
        """
        _response = await self._raw_client.email_auth(
            email=email,
            provider=provider,
            auth_type=auth_type,
            vital_link_token=vital_link_token,
            region=region,
            request_options=request_options,
        )
        return _response.data

    async def password_auth(
        self,
        *,
        username: str,
        password: str,
        provider: Providers,
        auth_type: AuthType,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Deprecated. Use `POST /v2/link/provider/password/{provider}` instead.

        Parameters
        ----------
        username : str

        password : str

        provider : Providers

        auth_type : AuthType

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import Providers
        from vital import AuthType
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.password_auth(username='username', password='password', provider=Providers.OURA, auth_type=AuthType.PASSWORD, )
        asyncio.run(main())
        """
        _response = await self._raw_client.password_auth(
            username=username,
            password=password,
            provider=provider,
            auth_type=auth_type,
            vital_link_token=vital_link_token,
            request_options=request_options,
        )
        return _response.data

    async def generate_oauth_link(
        self,
        oauth_provider: OAuthProviders,
        *,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Source:
        """
        This endpoint generates an OAuth link for oauth provider

        Parameters
        ----------
        oauth_provider : OAuthProviders

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Source
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import OAuthProviders
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.generate_oauth_link(oauth_provider=OAuthProviders.OURA, )
        asyncio.run(main())
        """
        _response = await self._raw_client.generate_oauth_link(
            oauth_provider, vital_link_token=vital_link_token, request_options=request_options
        )
        return _response.data

    async def connect_password_provider(
        self,
        provider: PasswordProviders,
        *,
        username: str,
        password: str,
        vital_link_token: typing.Optional[str] = None,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProviderLinkResponse:
        """
        This connects auth providers that are password based.

        Parameters
        ----------
        provider : PasswordProviders

        username : str
            Username for provider

        password : str
            Password for provider

        vital_link_token : typing.Optional[str]

        region : typing.Optional[Region]
            Provider region to authenticate against. Only applicable to specific providers.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProviderLinkResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import PasswordProviders
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.connect_password_provider(provider=PasswordProviders.WHOOP, username='username', password='password', )
        asyncio.run(main())
        """
        _response = await self._raw_client.connect_password_provider(
            provider,
            username=username,
            password=password,
            vital_link_token=vital_link_token,
            region=region,
            request_options=request_options,
        )
        return _response.data

    async def complete_password_provider_mfa(
        self,
        provider: PasswordProviders,
        *,
        mfa_code: str,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProviderLinkResponse:
        """
        This connects auth providers that are password based.

        Parameters
        ----------
        provider : PasswordProviders

        mfa_code : str

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProviderLinkResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import PasswordProviders
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.complete_password_provider_mfa(provider=PasswordProviders.WHOOP, mfa_code='mfa_code', )
        asyncio.run(main())
        """
        _response = await self._raw_client.complete_password_provider_mfa(
            provider, mfa_code=mfa_code, vital_link_token=vital_link_token, request_options=request_options
        )
        return _response.data

    async def connect_email_auth_provider(
        self,
        *,
        email: str,
        vital_link_token: typing.Optional[str] = None,
        email_provider_auth_link_provider: typing.Optional[Providers] = OMIT,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        This connects auth providers that are email based.

        Parameters
        ----------
        email : str

        vital_link_token : typing.Optional[str]

        email_provider_auth_link_provider : typing.Optional[Providers]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.connect_email_auth_provider(email='email', )
        asyncio.run(main())
        """
        _response = await self._raw_client.connect_email_auth_provider(
            email=email,
            vital_link_token=vital_link_token,
            email_provider_auth_link_provider=email_provider_auth_link_provider,
            region=region,
            request_options=request_options,
        )
        return _response.data

    async def get_all_providers(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[SourceLink]:
        """
        GET List of all available providers given the generated link token.

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceLink]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.get_all_providers()
        asyncio.run(main())
        """
        _response = await self._raw_client.get_all_providers(
            vital_link_token=vital_link_token, request_options=request_options
        )
        return _response.data

    async def connect_manual_provider(
        self,
        provider: ManualProviders,
        *,
        user_id: str,
        provider_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, bool]:
        """
        Parameters
        ----------
        provider : ManualProviders

        user_id : str

        provider_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, bool]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import ManualProviders
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.connect_manual_provider(provider=ManualProviders.BEURER_BLE, user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.connect_manual_provider(
            provider, user_id=user_id, provider_id=provider_id, request_options=request_options
        )
        return _response.data

    async def connect_demo_provider(
        self, *, user_id: str, provider: DemoProviders, request_options: typing.Optional[RequestOptions] = None
    ) -> DemoConnectionStatus:
        """
        POST Connect the given Vital user to a demo provider.

        Parameters
        ----------
        user_id : str
            Vital user ID

        provider : DemoProviders
            Demo provider. For more information, please check out our docs (https://docs.tryvital.io/wearables/providers/test_data)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DemoConnectionStatus
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import DemoProviders
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.link.connect_demo_provider(user_id='user_id', provider=DemoProviders.APPLE_HEALTH_KIT, )
        asyncio.run(main())
        """
        _response = await self._raw_client.connect_demo_provider(
            user_id=user_id, provider=provider, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.address import Address
from ..types.client_facing_payor import ClientFacingPayor
from ..types.payor_code_external_provider import PayorCodeExternalProvider
from .raw_client import AsyncRawPayorClient, RawPayorClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PayorClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPayorClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPayorClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPayorClient
        """
        return self._raw_client

    def create_payor(
        self,
        *,
        name: str,
        address: Address,
        provider: typing.Optional[PayorCodeExternalProvider] = OMIT,
        provider_payor_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingPayor:
        """
        Parameters
        ----------
        name : str

        address : Address

        provider : typing.Optional[PayorCodeExternalProvider]

        provider_payor_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingPayor
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import Address
        client = Vital(api_key="YOUR_API_KEY", )
        client.payor.create_payor(name='name', address=Address(first_line='first_line', country='country', zip='zip', city='city', state='state', ), )
        """
        _response = self._raw_client.create_payor(
            name=name,
            address=address,
            provider=provider,
            provider_payor_id=provider_payor_id,
            request_options=request_options,
        )
        return _response.data


class AsyncPayorClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPayorClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPayorClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPayorClient
        """
        return self._raw_client

    async def create_payor(
        self,
        *,
        name: str,
        address: Address,
        provider: typing.Optional[PayorCodeExternalProvider] = OMIT,
        provider_payor_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingPayor:
        """
        Parameters
        ----------
        name : str

        address : Address

        provider : typing.Optional[PayorCodeExternalProvider]

        provider_payor_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingPayor
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import Address
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.payor.create_payor(name='name', address=Address(first_line='first_line', country='country', zip='zip', city='city', state='state', ), )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_payor(
            name=name,
            address=address,
            provider=provider,
            provider_payor_id=provider_payor_id,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .get_user_historical_pulls_introspect_request_provider import GetUserHistoricalPullsIntrospectRequestProvider

__all__ = ["GetUserHistoricalPullsIntrospectRequestProvider"]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class LabTestCollectionMethod(str, enum.Enum):
    """
    The method used to perform a lab test. ℹ️ This enum is non-exhaustive.
    """

    TESTKIT = "testkit"
    WALK_IN_TEST = "walk_in_test"
    AT_HOME_PHLEBOTOMY = "at_home_phlebotomy"
    ON_SITE_COLLECTION = "on_site_collection"
    _UNKNOWN = "__LABTESTCOLLECTIONMETHOD_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "LabTestCollectionMethod":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        testkit: typing.Callable[[], T_Result],
        walk_in_test: typing.Callable[[], T_Result],
        at_home_phlebotomy: typing.Callable[[], T_Result],
        on_site_collection: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is LabTestCollectionMethod.TESTKIT:
            return testkit()
        if self is LabTestCollectionMethod.WALK_IN_TEST:
            return walk_in_test()
        if self is LabTestCollectionMethod.AT_HOME_PHLEBOTOMY:
            return at_home_phlebotomy()
        if self is LabTestCollectionMethod.ON_SITE_COLLECTION:
            return on_site_collection()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_facing_source import ClientFacingSource
from .client_facing_sport import ClientFacingSport
from .client_facing_workout_map import ClientFacingWorkoutMap


class ClientFacingWorkout(UniversalBaseModel):
    id: str
    user_id: str = pydantic.Field()
    """
    User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Title given for the workout
    """

    timezone_offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    Timezone offset from UTC as seconds. For example, EEST (Eastern European Summer Time, +3h) is 10800. PST (Pacific Standard Time, -8h) is -28800::seconds
    """

    average_hr: typing.Optional[int] = pydantic.Field(default=None)
    """
    Average heart rate during workout::bpm
    """

    max_hr: typing.Optional[int] = pydantic.Field(default=None)
    """
    Max heart rate during workout::bpm
    """

    distance: typing.Optional[float] = pydantic.Field(default=None)
    """
    Distance travelled during workout::meters
    """

    calendar_date: dt.date = pydantic.Field()
    """
    Date of the workout summary in the YYYY-mm-dd format. This generally matches the workout start date.
    """

    time_start: dt.datetime = pydantic.Field()
    """
    Start time of the workout::time
    """

    time_end: dt.datetime = pydantic.Field()
    """
    End time of the workout::time
    """

    calories: typing.Optional[float] = pydantic.Field(default=None)
    """
    Calories burned during the workout::kCal
    """

    sport: typing.Optional[ClientFacingSport] = pydantic.Field(default=None)
    """
    Sport's name
    """

    hr_zones: typing.Optional[typing.List[int]] = pydantic.Field(default=None)
    """
    Time in seconds spent in different heart rate zones <50%, 50-60%, 60-70%, 70-80%, 80-90%, 90%+. Due to rounding errors, it's possible that summing all values is different than the total time of the workout. Not available for all providers::seconds
    """

    moving_time: typing.Optional[int] = pydantic.Field(default=None)
    """
    Time spent active during the workout::seconds
    """

    total_elevation_gain: typing.Optional[float] = pydantic.Field(default=None)
    """
    Elevation gain during the workout::meters
    """

    elev_high: typing.Optional[float] = pydantic.Field(default=None)
    """
    Highest point of elevation::meters
    """

    elev_low: typing.Optional[float] = pydantic.Field(default=None)
    """
    Lowest point of elevation::meters
    """

    average_speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    Average speed during workout in m/s::meters/sec
    """

    max_speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    Max speed during workout in m/s::meters/sec
    """

    average_watts: typing.Optional[float] = pydantic.Field(default=None)
    """
    Average watts burned during exercise::watts
    """

    device_watts: typing.Optional[float] = pydantic.Field(default=None)
    """
    Watts burned during exercise::watts
    """

    max_watts: typing.Optional[float] = pydantic.Field(default=None)
    """
    Max watts burned during exercise::watts
    """

    weighted_average_watts: typing.Optional[float] = pydantic.Field(default=None)
    """
    Weighted average watts burned during exercise::watts
    """

    steps: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of steps accumulated during this workout::count
    """

    map_: typing.Optional[ClientFacingWorkoutMap] = pydantic.Field(alias="map", default=None)
    """
    Map of the workout
    """

    provider_id: str = pydantic.Field()
    """
    Provider ID given for that specific workout
    """

    source: ClientFacingSource = pydantic.Field()
    """
    Source the data has come from.
    """

    created_at: dt.datetime
    updated_at: dt.datetime

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
